local assert = assert

local game = {
  libs = {},
  states = {
    head = nil,
    title = {
      template = require 'states.title',
      parent = nil
    },
    credits = {
      template = require 'states.credits',
      parent = "title"
    },
    titleoptions = {
      template = require 'states.titleoptions',
      parent = "title"
    },
    ingame = {
      template = require 'states.ingame',
      parent = "title"
    },
    ingameoptions = {
      template = require 'states.ingameoptions',
      parent = "ingame"
    },
  }
}

local newState = function (blankState, stateTemplate)
  return stateTemplate(blankState)
end

function love.load ()
  love.graphics.setMode(love.graphics.getMode())
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.setBackgroundColor(39, 40, 34, 255)
  love.graphics.setFont(love.graphics.newFont(16))
  -- load libs
  game.libs.Gamestate = require 'lib.hump.gamestate'

  for name, value in pairs(game.states) do
    value.state = value.template(game.libs.Gamestate.new())
    value.state.parent = value.parent
  end

  game.states.head = game.states.ingame.state
  game.libs.Gamestate.switch(game.states.head)
end

function love.update (dt)
  local nextstate = game.states.head:update(dt)

  if nextstate then
    assert(game.states[nextstate] ~= nil, string.format("%s does not exist", nextstate))
    game.libs.Gamestate.switch(game.states[nextstate].state)
    game.states.head = game.states[nextstate].state
  end
end

function love.draw ()
  game.states.head:draw()
end

function love.keypressed (key, code)
  game.states.head:keypressed(key, code)
end
